<?php
/**
 * Auto_Google_News_Base_Job Class
 *
 * @since  1.0.0
 * @package Auto Google News
 */

if ( ! defined( 'ABSPATH' ) ) {
    exit;
} // Exit if accessed directly

if ( ! class_exists( 'Auto_Google_News_Base_Job' ) ) :

    abstract class Auto_Google_News_Base_Job {

        /**
         * Job ID
         *
         * @int
         */
        public $id;

        /**
         * Type
         *
         * @var string
         */
        public $type = '';

        /**
         * API data
         *
         * @var array
         */
        public $api_data = array();

        /**
         * Settings
         *
         * @var array
         */
        public $settings = array();

        /**
         * Keywords
         *
         * @var array
         */
        public $keywords = array();

        /**
         * Log Message
         *
         * @var array
         */
        public $log = array();

        /**
         * Run this job
         *
         * @return array
         */
        public function run(){

        }

        /**
         * Prepare Post Data
         * @param string $title
         * @param string $content
         * @return array
         * @since  1.0.0
         */
        public function prepare_post($title, $content, $settings = array()) {

            // Set words limit
            // if(isset($this->settings['robot_words_limit']) && !empty($this->settings['robot_words_limit'])){
            //      $content = substr($content, 0, $this->settings['robot_words_limit']);
            // }

            $post_data = array();
            $post_data['post_title']   = $title;
            $post_data['post_content'] = $content;
            $post_data['post_type']    = $settings['google_news_post_type'];
            $post_data['post_status']  = $settings['google_news_post_status'];
            $post_author = get_user_by('login',$settings['google_news_post_author']);
            $post_data['post_author']  = $post_author->ID;

            $post_data['post_category'] = $this->settings['google_news-post-category'];

            // get tags name
            $tags = array();
            $tag_ids = $this->settings['google_news-post-tag'];

            foreach($tag_ids as $tag_id){
                $tag = get_tag($tag_id);
                $tags[] = $tag->name;
            }

            $post_data['tags_input'] = array_values( $tags );;

            return $post_data;

        }

        /**
         * Create Post
         * @param string $title
         * @param string $content
         * @return array
         * @since  1.0.0
         */
        public function create_post($title, $content, $settings = array(), $image_url=''){

            // check errors
            $new_post = array('id' => false, 'error' => false, 'permalink' => '');

            if($this->is_title_duplicate($title)){
                $this->log[] = array(
                    'message'  => 'Title Duplicate ' . $title,
                    'level'    => 'log'
                );
                $new_post['error'] = 'post title duplicate.';

                return;
            }

            $post_data = $this->prepare_post($title, $content, $settings);

            $new_post['id'] = wp_insert_post($post_data);

            $post_id = isset($new_post['id']) ? $new_post['id'] : null;

            if (!$post_id) {
                $new_post['error'] = 'post-fail';
            }else{
                $new_post['permalink'] = get_permalink( $post_id );
                // Set post feature image
                $this->set_post_feature_image($post_id, $image_url);
            }

            if(false === $new_post['error']){
                $this->log[] = array(
                    'message'  => 'Post New Post Success: <a href="'.$new_post['permalink'].'">'.$title.'</a>',
                    'level'    => 'log'
                );
            }else{
                $this->log[] = array(
                    'message'  => 'Generate New Post Error: '.$new_post['error'],
                    'level'    => 'log'
                );
            }


        }

        /**
         * Set post feature image
         * @param int $parent_post_id
         * @return bool
         * @since  1.0.0
         */
        public function set_post_feature_image($post_id, $image_url){
            // Add Featured Image to Post
            $image_name       = 'wp-header-logo.png';
            $upload_dir       = wp_upload_dir(); // Set upload folder
            $image_data       = file_get_contents($image_url); // Get image data
            $unique_file_name = wp_unique_filename( $upload_dir['path'], $image_name ); // Generate unique name
            $filename         = basename( $unique_file_name ); // Create image file name

            // Check folder permission and define file location
            if( wp_mkdir_p( $upload_dir['path'] ) ) {
                $file = $upload_dir['path'] . '/' . $filename;
            } else {
                $file = $upload_dir['basedir'] . '/' . $filename;
            }

            // Create the image file on the server
            file_put_contents( $file, $image_data );

            // Check image file type
            $wp_filetype = wp_check_filetype( $filename, null );

            // Set attachment data
            $attachment = array(
                'post_mime_type' => $wp_filetype['type'],
                'post_title'     => sanitize_file_name( $filename ),
                'post_content'   => '',
                'post_status'    => 'inherit'
            );

            // Create the attachment
            $attach_id = wp_insert_attachment( $attachment, $file, $post_id );

            // Include image.php
            require_once(ABSPATH . 'wp-admin/includes/image.php');

            // Define attachment metadata
            $attach_data = wp_generate_attachment_metadata( $attach_id, $file );

            // Assign metadata to attachment
            wp_update_attachment_metadata( $attach_id, $attach_data );

            // And finally assign featured image to post
            set_post_thumbnail( $post_id, $attach_id );
        }

        /**
         * check if post title already exists
         * @param string $title
         * @return bool
         * @since  1.0.0
         */
        public function is_title_duplicate($title){
            if( get_page_by_title( $title, 'OBJECT', 'post' )  ){
                return true;
            }else{
                return false;
            }
        }

        /**
         * Get API Data.
         *
         * @since 1.0.0
         */
        public function get_api_data($type) {
            $this->api_data = Auto_Google_News_Addon_Loader::get_instance()->get_addon_data($type);
        }

        /**
        * Fetch stream bt HTTP GET Method
        *
        * @param  string $url
        * @return string
        */
        public function fetch_stream( $url, $headers = array()) {
          // build http request args
          $args = array(
              'headers' => $headers,
              'timeout'     => '20'
          );

          $request = wp_remote_get( $url, $args );

          // retrieve the body from the raw response
          $json_posts = wp_remote_retrieve_body( $request );

          // log error messages
          if ( is_wp_error( $request ) ) {
              $this->log[] = array(
                'message'  => 'Fetching failed with WP_Error: '. $request->errors['http_request_failed'][0],
                'level'    => 'log'
            );
              return $request;
          }

          if ( $request['response']['code'] != 200 ) {
              $this->log[] = array(
                'message'  => 'Fetching failed with code: ' . $request['response']['code'],
                'level'    => 'log'
            );
              return false;
          }

          return $json_posts;
        }

        /**
         * Fetch stream bt HTTP POST Method
         *
         * @param  string $url
         * @return string
         */
        public function fetch_post( $url, $headers = array(), $body = array()) {
            // build http request args
            $args = array(
                'headers' => $headers,
                'body'    => $body,
                'method'  => 'POST',
                'timeout' => 45,
            );

            $request = wp_remote_post( $url, $args );

            // retrieve the body from the raw response
            $json_posts = wp_remote_retrieve_body( $request );

            // log error messages
            if ( is_wp_error( $request ) ) {
                $this->log[] = array(
                    'message'  => 'Fetching failed with WP_Error: '. $request->errors['http_request_failed'][0],
                    'level'    => 'log'
                );
                return $request;
            }

            if ( $request['response']['code'] != 200 ) {
                $this->log[] = array(
                    'message'  => 'Fetching failed with code: ' . $request['response']['code'],
                    'level'    => 'log'
                );
                return false;
            }

            return $json_posts;

        }

    }

endif;
