<?php
/**
 * Auto_Google_News_Job Class
 *
 * @since  1.0.0
 * @package Auto Google News
 */

if ( ! defined( 'ABSPATH' ) ) {
    exit;
} // Exit if accessed directly

if ( ! class_exists( 'Auto_Google_News_Job' ) ) :

    class Auto_Google_News_Job extends Auto_Google_News_Base_Job{

        /**
         * Feed Link
         *
         * @var string
         */
        public $feed_link = '';

        /**
         * Auto_Google_News_Job constructor.
         *
         * @since 1.0.0
         */
        public function __construct($id, $keyword, $settings) {
            $this->id = $id;
            $this->keyword = $keyword;
            $this->settings = $settings;
            $this->logger = new Auto_Google_News_Log($id);
            $this->log = array();
        }

        /**
         * Run this job
         *
         * @return array
         */
        public function run(){
            $response = array();

            // Fetch Data
            $data = $this->fetch_data();

            if(is_object($data)){
                $this->log[] = array(
                    'message'   => $data->errors['simplepie-error'][0],
                    'level'     => 'error'
                );
            }else{
                $title = $data['title'];

                // Build post content
                $content = $data['content'].'<br>';
                $content .= '<a href="'.$data['link'].'">source</a>';

                // Generate New Post
                $this->create_post($title, $content, $this->settings, $data['featured_image']);
            }

            // Add this job running log to system log file
            foreach($this->log as $key => $value){
                $this->logger->add($value['message'], $value['level']);
                $response[$key] = $value['message'];
            }

            return $response;
        }

        /**
        * Fetch Data
        *
        * @return array
        */
        public function fetch_data() {
            $google_news_init_language = str_replace('_', ':' ,$this->settings['google_news_init_language']);
            $pieces = explode(":", $google_news_init_language);
            $country_code = $pieces[0];
            $language_code = $pieces[1];

            // Search google news feed by keyword
            $url = 'https://news.google.com/rss/search?';
            $req_params = [
                'q' => $this->keyword,
                'hl' => $language_code,
                'gl' => $country_code,
                'ceid' => $google_news_init_language,
            ];

            $url .= http_build_query($req_params);

            // Get RSS Feed(s)
            include_once( ABSPATH . WPINC . '/feed.php' );

            // Get a SimplePie feed object from the specified feed source.
            $rss = fetch_feed( $url );

            $maxitems = 0;

            // Checks that the object is created correctly
            if ( ! is_wp_error( $rss ) ) {

                // Figure out how many total items there are, but limit it to 5.
                $maxitems = $rss->get_item_quantity();

                $this->log[] = array(
                    'message'   => 'Feed contains '.$maxitems.' total items',
                    'level'     => 'log'
                );

                // Build an array of all the items, starting with element 0 (first element).
                $rss_items = $rss->get_items(0, $maxitems);

                $rand = rand(0, $maxitems-1);

                $return = array();

                // translate title
                $return['title'] = $rss_items[$rand]->get_title();
                $return['link'] = $rss_items[$rand]->get_permalink();

                // Fetch origin content
                $original_content  = $this->fetch_stream( $return['link'] );

                $return['featured_image'] = $this->get_og_image($original_content);

                if(!empty($original_content)){
                    // Parse DOM HTML
                    // Create DOM from URL or file
                    $html = str_get_html($original_content);
                    if($html){
                        $main_content = '';
                        // get each paragraph
                        foreach($html->find('p') as $element) {
                            $innertext = $element->innertext;
                            $main_content .= $innertext . '<br>';
                        }
                        $return['content'] = $main_content;
                    }
                }else{
                    $return['content'] = $rss_items[$rand]->get_content();
                }

                $this->log[] = array(
                    'message'   => 'Fetch Title: '.$return['title'],
                    'level'     => 'log'
                );
                $this->log[] = array(
                    'message'   => 'Fetch URL: <a href="'.$return['link'].'">'.$return['link'].'</a>',
                    'level'     => 'log'
                );

                return $return;
            }else{
                return $rss;
            }

        }

        /**
         * Get og:image source
         *
         * @return string
         */
        private function get_og_image($original_content){

            $og_img = '';

            // let's find og:image may be the content we got has no image
            preg_match ( '{<meta[^<]*?(?:property|name)=["|\']og:image["|\'][^<]*?>}s', $original_content, $plain_og_matches );

            if (isset ( $plain_og_matches [0] ) && stristr ( $plain_og_matches [0], 'og:image' )) {
                preg_match ( '{content=["|\'](.*?)["|\']}s', $plain_og_matches [0], $matches );
                $og_img = $matches [1];
            }

            return $og_img;
        }
}

endif;
